"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = require("chalk");
const codestore_utils_1 = require("codestore-utils");
const command_1 = require("@oclif/command");
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const listr2_1 = require("listr2");
const db_command_1 = tslib_1.__importDefault(require("../../lib/db.command"));
class Import extends db_command_1.default {
    async execute() {
        const { flags: flagsData, args: { table, path } } = this.parse(Import);
        const db = await super.getDb(flagsData);
        if (!db) {
            return;
        }
        const { environment, service, project } = db;
        let { append } = flagsData;
        if (!append) {
            const confirmed = (await inquirer_1.default.prompt([{
                    name: 'result',
                    message: 'Are you sure you want to drop the table and upload new data to it?',
                    type: 'confirm',
                    default: false,
                }])).result;
            if (!confirmed) {
                return;
            }
        }
        this.log(`Importing the data to table ${chalk_1.blue(table)}...`);
        const [header, ...data] = (await codestore_utils_1.PromisifiedFs.readFile(path, 'utf8')).split('\n');
        const totalRows = data.length;
        let importedRows = 0;
        const partLength = 75000;
        let result = { inserted: 0, sql: '' };
        const tasks = new listr2_1.Listr([
            {
                title: `Imported 0 of ${totalRows} rows`,
                task: async (ctx, task) => {
                    while (data.length) {
                        let part = header;
                        while (data.length && part.length + data[0].length < partLength) {
                            part += `\n${data.shift()}`;
                        }
                        // eslint-disable-next-line no-await-in-loop
                        result = await this.codestore.Database.importTable(service.id, environment, project.id, table, part, append);
                        importedRows += result.inserted;
                        if (result.errorMessage) {
                            return;
                        }
                        append = true;
                        task.title = `Imported ${importedRows} of ${totalRows} rows`;
                    }
                },
            },
        ]);
        await tasks.run();
        if (result.errorMessage) {
            this.log(`An error occured for row ${importedRows + 1} (line ${importedRows + 2} at ${path})`);
            this.log(`while executing: ${result.sql}`);
            this.error(result.errorMessage);
        }
    }
}
exports.default = Import;
Import.flags = Object.assign(Object.assign({}, db_command_1.default.flags), { append: command_1.flags.boolean({
        char: 'a',
        description: 'Append the data to the table instead of replacing it',
    }) });
Import.args = [
    { name: 'table', required: true, description: '(required) Table name' },
    { name: 'path', required: false, description: '(required) Relative path from the service\'s root to the csv file with table\'s data' },
];
Import.description = 'Import the data from csv file to a table from the service\'s database (by default it drops the data first)';
