"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const listr2_1 = require("listr2");
const project_service_billing_type_enum_1 = tslib_1.__importDefault(require("../../../common/constants/project-service-billing-type.enum"));
const utils_1 = require("../../../common/utils");
const command_2 = tslib_1.__importDefault(require("../../../lib/command"));
class Add extends command_2.default {
    async execute() {
        const { args, flags: flagsData } = this.parse(Add);
        let project;
        let service;
        let billingType = project_service_billing_type_enum_1.default.NOT_BILLABLE;
        let billingValue = '';
        if (this.flagsLength(flagsData)) {
            const data = await this.validateFlags(Add.flags, flagsData);
            project = await this.findProject(data.project);
            if (!project) {
                this.error(`Project with id ${chalk_1.blue(data.project)} does not exist`);
            }
            service = await this.findService(data.service);
            if (!service) {
                this.error(`Service with id ${chalk_1.blue(data.service)} does not exist`);
            }
            if (data.billingType) {
                billingType = data.billingType;
                billingValue = data.billingValue;
            }
        }
        else {
            project = await this.chooseProject(args, 'Choose a project you want to add to');
            if (!project)
                return;
            service = await this.chooseService(args, 'Choose a service which you want to add');
            if (!service)
                return;
            const { wantToBill } = await inquirer_1.default.prompt({
                type: 'confirm',
                name: 'wantToBill',
                message: 'Enable billing:',
                prefix: utils_1.createPrefix('Would you like to enable billing for your service?'),
            });
            if (wantToBill) {
                const userFriendlyBillingPlans = {
                    'Pay Per Call': project_service_billing_type_enum_1.default.PAY_PER_CALL,
                    'Monthly payments': project_service_billing_type_enum_1.default.MONTHLY,
                };
                const { howToBill } = await inquirer_1.default.prompt({
                    type: 'list',
                    name: 'howToBill',
                    message: 'Billing plan:',
                    choices: Object.keys(userFriendlyBillingPlans),
                    prefix: utils_1.createPrefix('Choose a billing plan for your service'),
                });
                billingType = userFriendlyBillingPlans[howToBill];
                billingValue = (await inquirer_1.default.prompt({
                    type: 'input',
                    validate: (value) => {
                        if (!value) {
                            return 'Billing value is required';
                        }
                        return true;
                    },
                    name: 'billingValue',
                    message: 'Billing value:',
                    prefix: utils_1.createPrefix('Enter a billing value for your service'),
                })).billingValue;
            }
        }
        const projectUniqueName = project.uniqueName;
        const serviceUniqueName = service.uniqueName;
        await new listr2_1.Listr([{
                title: `Including service with id ${chalk_1.blue(serviceUniqueName)} to project ${chalk_1.blue(projectUniqueName)}`,
                task: async (ctx, task) => {
                    try {
                        const { status } = await this.codestore.Project.includeServiceByUniqueName(projectUniqueName, serviceUniqueName, billingType, billingValue);
                        task.title = `Service ${chalk_1.blue(serviceUniqueName)} is included to project ${chalk_1.blue(projectUniqueName)}\nStatus: ${chalk_1.bold(status)}`;
                    }
                    catch (error) {
                        if (error.message.match(/already includes/)) { // TODO: should be changed to error status code check later, when backend will support it.
                            task.title = `Project ${chalk_1.blue(projectUniqueName)} already includes service ${chalk_1.blue(serviceUniqueName)}`;
                        }
                        else
                            throw error;
                    }
                },
            }]).run();
    }
    get rules() {
        return {
            service: (value) => {
                if (!value) {
                    return 'ID of the service is required';
                }
                return true;
            },
            project: (value) => {
                if (!value) {
                    return 'ID of the project is required';
                }
                return true;
            },
            billingType: (value) => {
                const billingTypes = [project_service_billing_type_enum_1.default.MONTHLY, project_service_billing_type_enum_1.default.PAY_PER_CALL];
                if (value && !billingTypes.includes(value)) {
                    return `Billing type should be one of: ${billingTypes.join(', ')}`;
                }
                return true;
            },
        };
    }
}
exports.default = Add;
Add.description = 'Adds and existing service to your project';
Add.args = [
    { name: 'projectArg', description: 'ID of the project' },
    { name: 'serviceArg', description: 'ID of the service' },
];
Add.flags = Object.assign(Object.assign({}, command_2.default.flags), { project: command_1.flags.string({
        char: 'p',
        description: 'Project name',
    }), service: command_1.flags.string({
        char: 's',
        description: 'Service name',
    }), billingType: command_1.flags.string({
        char: 't',
        description: 'Billing type',
        dependsOn: ['billingValue'],
    }), billingValue: command_1.flags.string({
        char: 'v',
        description: 'Billing value',
        dependsOn: ['billingType'],
    }) });
