"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const command_2 = tslib_1.__importDefault(require("../../../lib/command"));
const environment_enum_1 = require("../../../common/constants/environment.enum");
class Promote extends command_2.default {
    async execute() {
        const { args, flags: flagsData } = this.parse(Promote);
        let project;
        let service;
        let targetEnvironment;
        let sourceEnvironment;
        if (this.flagsLength(flagsData)) {
            const data = await this.validateFlags(Promote.flags, flagsData);
            project = await this.findProject(data.project);
            if (!project) {
                this.error(`Project with id ${chalk_1.blue(data.project)} does not exist`);
            }
            service = await this.findService(data.service, project.projectServices.map((ps) => ps.service));
            if (!service) {
                this.error(`Service with id ${chalk_1.blue(data.service)} does not exist in project ${chalk_1.blue(project.uniqueName)}`);
            }
            targetEnvironment = data.targetEnvironment;
            sourceEnvironment = data.sourceEnvironment;
        }
        else {
            project = await this.chooseProject(args, 'Choose a project you want to promote to');
            if (!project)
                return;
            service = await this.chooseService(args, 'Choose a service which you want to promote', project.projectServices.map((ps) => ps.service));
            if (!service)
                return;
            const deployments = await this.codestore.Deployment.getDeploymentsForProjectService(project.id, service.id);
            const sourceEnvironments = new Map(deployments.map((d) => Promote.mapDeploymentToEnvAndVersion(d)));
            sourceEnvironment = await this.chooseEnvironment(sourceEnvironments, 'Please select source environment');
            const environments = new Map(environment_enum_1.projectServiceEnvironments
                .filter((e) => e !== sourceEnvironment)
                .map((e) => {
                const deployment = deployments.find((d) => d.environment.name === e);
                return Promote.mapDeploymentToEnvAndVersion(deployment, e);
            }));
            targetEnvironment = await this.chooseEnvironment(environments, 'Please select target environment');
        }
        const projectUniqueName = project.uniqueName;
        const serviceUniqueName = service.uniqueName;
        this.log(); // print a blank line
        try {
            await this.codestore.Project.promoteServiceByUniqueName({
                projectUniqueName,
                serviceUniqueName,
                sourceEnvironment,
                targetEnvironment,
            });
            this.log(`Updated "${chalk_1.bold(targetEnvironment)}" in project ${chalk_1.blue(projectUniqueName)}`);
        }
        catch (error) {
            if (error.message.match(/is up to date/)) {
                this.log(`\n"${chalk_1.bold(targetEnvironment)}" environment of service ${chalk_1.blue(serviceUniqueName)} is up to date`);
            }
        }
    }
    static mapDeploymentToEnvAndVersion(deployment, environment) {
        if (!deployment && !environment)
            throw new Error('One of arguments should be provided');
        if (deployment)
            return [`${deployment.environment.name} v${deployment.version.name}`, deployment.environment.name];
        return [environment, environment];
    }
    get rules() {
        return {
            service: (value) => {
                if (!value) {
                    return 'ID of the service is required';
                }
                return true;
            },
            project: (value) => {
                if (!value) {
                    return 'ID of the project is required';
                }
                return true;
            },
            targetEnvironment: (value) => {
                if (!value) {
                    return 'Target environment is required';
                }
                return true;
            },
            sourceEnvironment: (value) => {
                if (!value) {
                    return 'Source environment is required';
                }
                return true;
            },
        };
    }
}
exports.default = Promote;
Promote.description = `Promotes service inside the project between Development, Staging and Production environments.
Run without arguments to select project and service from a list.`;
Promote.flags = Object.assign(Object.assign({}, command_2.default.flags), { project: command_1.flags.string({
        char: 'p',
        description: 'ID of the project',
    }), service: command_1.flags.string({
        char: 's',
        description: 'ID of the service',
    }), sourceEnvironment: command_1.flags.string({
        char: 'e',
        options: environment_enum_1.projectServiceEnvironments,
        description: 'Source environment',
    }), targetEnvironment: command_1.flags.string({
        char: 't',
        options: environment_enum_1.projectServiceEnvironments,
        description: 'Target environment',
    }) });
Promote.args = [
    { name: 'projectArg', description: 'ID of the project' },
    { name: 'serviceArg', description: 'ID of the service' },
];
