"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const listr2_1 = require("listr2");
const tree_node_cli_1 = tslib_1.__importDefault(require("tree-node-cli"));
const environment_enum_1 = require("../../common/constants/environment.enum");
const string_length_enum_1 = tslib_1.__importDefault(require("../../common/constants/string.length.enum"));
const file_worker_1 = tslib_1.__importDefault(require("../../common/file-worker"));
const utils_1 = require("../../common/utils");
const command_2 = tslib_1.__importDefault(require("../../lib/command"));
const ServiceInfo = tslib_1.__importStar(require("./info"));
const template_enum_1 = tslib_1.__importDefault(require("../../common/constants/template.enum"));
class Create extends command_2.default {
    constructor() {
        super(...arguments);
        this.businessDomains = [];
    }
    async execute() {
        const { flags: flagsData } = this.parse(Create);
        let service;
        const { sourceType } = await inquirer_1.default.prompt([
            {
                name: 'sourceType',
                type: 'list',
                message: 'Select the source for your service',
                choices: ['code.store', 'github', 'gitlab'],
            },
        ]);
        if (sourceType !== 'code.store') {
            const list = await this.codestore.GitProviders.listUsersProjects(sourceType);
            const { chosenRepo } = await inquirer_1.default.prompt([{
                    name: 'chosenRepo',
                    type: 'list',
                    message: 'Select the source for your service',
                    choices: list.map((it) => it.name),
                }]);
            const { url } = list.find((it) => it.name === chosenRepo);
            this.externalGitUrl = url;
        }
        this.businessDomains = await this.codestore.Service.businessDomains();
        if (this.flagsLength(flagsData)) {
            service = await this.validateFlags(Create.flags, flagsData);
        }
        else {
            service = await inquirer_1.default.prompt([
                {
                    name: 'name',
                    message: 'Service name:',
                    validate: async (value) => this.validate('name', value),
                },
                {
                    name: 'problem',
                    message: 'What problem are you solving?:',
                    validate: async (value) => this.validate('problem', value),
                },
                {
                    name: 'howSolving',
                    message: 'How do you solve it?:',
                    validate: async (value) => this.validate('howSolving', value),
                    prefix: utils_1.createPrefix(`Describe how do you solve it? It's optional too and should look something like:
          This service provides an API to create, update and delete rooms and
          another set of queries to manage bookings, cancellations, and search for available rooms.`),
                },
                {
                    type: 'list',
                    name: 'businessDomain',
                    message: 'Business domain:',
                    choices: this.businessDomains,
                    prefix: utils_1.createPrefix(`What is the most relevant business domain of your service?
Use up/down arrows to navigate and hit ENTER to select.
Please select 'Other' as last option`),
                },
                {
                    name: 'tags',
                    message: 'Hashtags:',
                    validate: async (value) => this.validate('tags', value),
                    prefix: utils_1.createPrefix(`Now, the last thing, enter free-hashtags describing your service.
Up to 5, comma-separated, no need to add #.
Example:
hospitality, booking, meeting-rooms, office`),
                },
            ]);
        }
        const tasks = new listr2_1.Listr([{
                title: `Creating service "${chalk_1.bold(service.name)}"`,
                task: async (ctx, task) => {
                    const { service: { id, displayName: createdServiceName, uniqueName, developerKey, }, commitId, } = await this.codestore.Service.create({
                        name: service.name,
                        tags: service.tags,
                        problemSolving: service.problem,
                        howSolving: service.howSolving,
                        businessDomain: service.businessDomain.replace(' ', '_'),
                        private: true,
                        template: service.template || template_enum_1.default.Typescript,
                        url: this.externalGitUrl,
                    });
                    ctx.service = {
                        id,
                        createdServiceName,
                        commitId,
                        uniqueName,
                    };
                    this.serviceId = id;
                    this.developerKey = developerKey;
                    task.title = `Created service with ID: ${chalk_1.blue(uniqueName)}`;
                },
                options: { persistentOutput: true },
            },
            {
                title: 'Building the containers for private and demo environments',
                task: async (ctx, task) => {
                    await this.codestore.Service.checkServiceCreated(ctx.service.id);
                    task.title = 'Private and demo environment containers were built';
                },
            },
            {
                title: 'Deploying to private and demo environments',
                task: async (ctx, task) => {
                    await this.codestore.Service.checkServiceDeployed(ctx.service.id);
                    task.title = 'Deployed to private and demo environments';
                },
            },
            {
                title: 'Downloading service template',
                task: async (ctx) => {
                    const { id, uniqueName } = ctx.service;
                    const data = await this.codestore.Service.download(id);
                    await file_worker_1.default.saveZipFromB64(data, uniqueName);
                    this.structure = tree_node_cli_1.default(uniqueName, { exclude: [/node_modules/, /dist/] });
                },
            },
        ]);
        await tasks.run();
        const deployments = await this.codestore.Deployment.getDeploymentsForService(this.serviceId);
        const deploymentTo = ServiceInfo.deploymentsToEnvironments(deployments, environment_enum_1.serviceEnvironments);
        this.log(`Use developer key for private environment: ${chalk_1.bold(this.developerKey)}`);
        Object.keys(deploymentTo).forEach((environment) => {
            this.log(`Your service on ${environment} environment is available by this url: ${chalk_1.blue(command_2.default.getServiceUrl(deploymentTo[environment]))}`);
        });
        this.log('\n');
        this.log(chalk_1.yellow(this.structure));
    }
    get rules() {
        return {
            name: async (value) => {
                const name = 'Service name';
                if (!value.length) {
                    return `${name} should not be empty`;
                }
                if (value.length >= string_length_enum_1.default.MEDIUM) {
                    return `${name} for this field should be less than ${string_length_enum_1.default.MEDIUM} characters`;
                }
                if (value.length < string_length_enum_1.default.MIN) {
                    return `${name} must be longer than or equal to ${string_length_enum_1.default.MIN} characters`;
                }
                const available = await this.codestore.Service.isUniqueNameAvailable(value);
                if (available.free) {
                    return true;
                }
                return `Service with this name already exists in your organization, available names: ${available.variants.join(', ')}`;
            },
            problem: (value) => {
                const name = 'Problem solving';
                if (!value) {
                    return `${name} should be provided.`;
                }
                if (value.length >= string_length_enum_1.default.LONG) {
                    return `${name} should be less than ${string_length_enum_1.default.LONG} characters.`;
                }
                return true;
            },
            howSolving: (value) => {
                if (!value) {
                    return 'Describe how you solve it, please.';
                }
                if (value.length >= string_length_enum_1.default.LONG) {
                    return `Solving description should be less than ${string_length_enum_1.default.LONG} characters.`;
                }
                return true;
            },
            tags: (value) => {
                if (!value) {
                    return 'Tags should be provided.';
                }
                const tags = value.split(',');
                if (tags.length > string_length_enum_1.default.SERVICE_TAGS_COUNT) {
                    return `Please select a maximum of ${string_length_enum_1.default.SERVICE_TAGS_COUNT} tags.`;
                }
                const exception = tags.find((tag) => tag.length > string_length_enum_1.default.SERVICE_TAGS_MAX);
                if (exception) {
                    return `Tag ${exception} is more than ${string_length_enum_1.default.SERVICE_TAGS_MAX} characters`;
                }
                return true;
            },
            tempalte: (value) => {
                const name = 'Tempalte';
                if (!value) {
                    return `${name} should be provided.`;
                }
                const validTemplates = Object.values(template_enum_1.default);
                const isValid = validTemplates.includes(value);
                if (!isValid) {
                    return `${name} should be one of: ${validTemplates.join(', ')}`;
                }
                return true;
            },
            businessDomain: async (value) => {
                const name = 'Business domain';
                if (!value) {
                    return `${name} should be provided.`;
                }
                const isValid = this.businessDomains.includes(value);
                if (!isValid) {
                    return `${name} should be one of: ${this.businessDomains.join(', ')}`;
                }
                return true;
            },
        };
    }
}
exports.default = Create;
Create.flags = Object.assign(Object.assign({}, command_2.default.flags), { name: command_1.flags.string({
        char: 'n',
        description: 'Service name',
    }), problem: command_1.flags.string({
        char: 'p',
        description: 'What problem are you solving?',
    }), howSolving: command_1.flags.string({
        char: 's',
        description: 'How do you solve it?',
    }), businessDomain: command_1.flags.string({
        char: 'b',
        description: 'What is the most relevant business domain of your service?',
    }), tags: command_1.flags.string({
        char: 't',
        description: 'Tags (comma-separated)',
    }) });
Create.description = 'Create new service';
