"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const codestore_utils_1 = require("codestore-utils");
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const listr2_1 = require("listr2");
const resolver_validator_1 = tslib_1.__importDefault(require("../../common/resolver-validator"));
const aliases_1 = tslib_1.__importDefault(require("../../common/constants/aliases"));
const deployment_status_enum_1 = tslib_1.__importDefault(require("../../common/constants/deployment-status.enum"));
const file_worker_1 = tslib_1.__importDefault(require("../../common/file-worker"));
const utils_1 = require("../../common/utils");
const child_cli_1 = require("../../lib/child-cli");
const command_2 = tslib_1.__importDefault(require("../../lib/command"));
const template_enum_1 = tslib_1.__importDefault(require("../../common/constants/template.enum"));
class Push extends command_2.default {
    constructor() {
        super(...arguments);
        this.includeDescription = false;
        this.splitNotes = (notes) => notes.split(';');
        this.tasksForTemplate = {
            [template_enum_1.default.Typescript]: [
                {
                    title: 'Compiling your code',
                    task: async () => {
                        await child_cli_1.installDependencies();
                        await codestore_utils_1.compile(await codestore_utils_1.PathsResolverTool.loadFilesToCompile());
                    },
                },
                {
                    title: 'Validating schema',
                    task: async () => {
                        await codestore_utils_1.GraphqlValidatorTool.validateSchema();
                        const validationData = await codestore_utils_1.GraphqlValidatorTool.validateQueriesAndMutations();
                        await resolver_validator_1.default(validationData);
                    },
                },
                {
                    title: 'Preparing the service code for upload',
                    task: async (ctx) => {
                        await codestore_utils_1.PromisifiedFs.createFolderIfNotExist(codestore_utils_1.PathsResolverTool.MIGRATIONS);
                        await codestore_utils_1.PromisifiedFs.createFolderIfNotExist(codestore_utils_1.PathsResolverTool.ENTITIES);
                        await codestore_utils_1.PromisifiedFs.rimraf(codestore_utils_1.PathsResolverTool.BUILD);
                        ctx.encodedZip = await file_worker_1.default.zipFolder();
                    },
                },
            ],
            [template_enum_1.default.Golang]: [
                {
                    title: 'Validating schema',
                    task: async () => {
                        await codestore_utils_1.GraphqlValidatorTool.validateSchema();
                    },
                },
                {
                    title: 'Preparing the service code for upload',
                    task: async (ctx) => {
                        ctx.encodedZip = await file_worker_1.default.zipFolder();
                    },
                },
            ],
        };
    }
    async execute() {
        const { flags: flagsData } = this.parse(Push);
        const { serviceId, template } = await this.serviceWorker.loadValuesFromYaml();
        let releaseNotes;
        let confirmed = flagsData.force;
        if (!confirmed) {
            const schema = (await codestore_utils_1.PromisifiedFs.readFile(codestore_utils_1.PathsResolverTool.SCHEMA)).toString();
            const changes = await this.codestore.Service.compareSchemas(schema, serviceId)
                .catch((e) => { throw new Error(`Schema validation failed. ${e.message}`); });
            if (changes.length) {
                this.log(chalk_1.bold(chalk_1.red('Breaking changes were found between schemas')));
                this.renderTable(changes, {
                    type: {},
                    message: {},
                    criticalityLevel: {
                        header: 'Criticality',
                    },
                });
                confirmed = (await inquirer_1.default.prompt([{
                        name: 'result',
                        message: 'Are you sure you want to push new schema to private environment?',
                        type: 'confirm',
                        default: false,
                    }])).result;
            }
            else {
                confirmed = true;
            }
        }
        if (!confirmed)
            return;
        if (this.flagsLength(flagsData)) {
            const data = await this.validateFlags(Push.flags, flagsData);
            releaseNotes = data.releaseNotes;
        }
        else {
            const questions = this.getQuestions();
            const promptResult = await inquirer_1.default.prompt(questions);
            releaseNotes = promptResult.releaseNotes;
        }
        const deployment = await this.codestore.Deployment.getDeployment(serviceId);
        if (!deployment) {
            this.log(`Service with id ${chalk_1.blue(serviceId)} was not deployed`);
            return;
        }
        if (deployment.status === deployment_status_enum_1.default.DEPLOYMENT_INITIALIZED || deployment.status === deployment_status_enum_1.default.DEPLOYMENT_IN_PROGRESS) {
            this.log('Your service is deploying, please try again later.');
            return;
        }
        await this.push(template, serviceId, releaseNotes);
    }
    getQuestions() {
        const questions = [
            {
                name: 'releaseNotes',
                message: 'Notes:',
                validate: (value) => this.validate('releaseNotes', value),
                prefix: utils_1.createPrefix('Please enter release notes (semicolon separated)'),
            },
        ];
        if (this.includeDescription) {
            questions.unshift({
                name: 'description',
                message: 'Description:',
                validate: (value) => this.validate('description', value),
                prefix: utils_1.createPrefix('Please enter a short description of your changes'),
            });
        }
        return questions;
    }
    get rules() {
        return {
            description: (value) => {
                if (!value || this.splitNotes(value).length < 1) {
                    return 'At least one description note is required';
                }
                return true;
            },
            releaseNotes: (value) => {
                if (!value || this.splitNotes(value).length < 1) {
                    return 'At least one release note is required';
                }
                return true;
            },
        };
    }
    async push(template, serviceId, releaseNotes) {
        const tasks = new listr2_1.Listr([
            ...this.tasksForTemplate[template],
            {
                title: 'Pushing service',
                task: async (ctx, task) => {
                    const { encodedZip } = ctx;
                    await this.codestore.Service.push(encodedZip, this.splitNotes(releaseNotes), releaseNotes);
                    task.title = 'Service has been pushed';
                },
            },
            {
                title: 'Deploying to private environment',
                task: async (_, task) => {
                    await this.codestore.Deployment.checkDeployment(serviceId);
                    task.title = 'Deployed to private environment';
                },
            },
        ]);
        this.log('');
        await tasks.run();
    }
}
exports.default = Push;
Push.flags = Object.assign(Object.assign({}, command_2.default.flags), { releaseNotes: command_1.flags.string({
        char: 'r',
        description: 'Release notes (semicolon separated)',
    }), force: command_1.flags.boolean({
        char: 'f',
        description: 'Push despite of breaking or dangerous changes',
    }) });
Push.description = 'Push local changes to Private environment';
Push.aliases = [aliases_1.default.PUSH];
