"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const apollo_boost_1 = tslib_1.__importDefault(require("apollo-boost"));
const chalk_1 = require("chalk");
const joi_1 = tslib_1.__importDefault(require("joi"));
const cli_ux_1 = tslib_1.__importDefault(require("cli-ux"));
const cross_fetch_1 = tslib_1.__importDefault(require("cross-fetch"));
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const commandIds_1 = tslib_1.__importDefault(require("../common/constants/commandIds"));
const utils_1 = require("../common/utils");
const api_client_1 = tslib_1.__importDefault(require("./api-client"));
const errors_1 = require("./errors");
const home_folder_service_1 = tslib_1.__importDefault(require("./home-folder-service"));
const service_worker_1 = tslib_1.__importDefault(require("./service-worker"));
const verbose_logger_1 = tslib_1.__importDefault(require("./verbose-logger"));
const pjson = require('../../package.json');
class Command extends command_1.Command {
    constructor() {
        super(...arguments);
        this.homeFolderService = new home_folder_service_1.default();
        this.serviceWorker = new service_worker_1.default();
        this.base = `${pjson.name}@${pjson.version}`;
        this.authToken = process.env.AUTH_TOKEN;
    }
    get rules() {
        return {};
    }
    get codestore() {
        return this._codestore;
    }
    // do not override this method because it uses execute method to provide base error handling logic.
    async run() {
        const { flags: { verbose, verbosePath } } = this.parse(this.constructor);
        this.verboseLogger = new verbose_logger_1.default(verbose, verbosePath || this.homeFolderService.logsPath);
        try {
            await this.setupApiClient(this.id === commandIds_1.default.LOGIN || this.id === commandIds_1.default.DEV);
            this._codestore = new api_client_1.default(this.homeFolderService, this.gqlClient, this.verboseLogger.register(this.id || 'undefined command'));
            await this.execute();
        }
        catch (e) {
            if ((e === null || e === void 0 ? void 0 : e.constructor) === errors_1.GeneratorError) {
                this.error(`${e.message}\nYou can find the full generation log here: ${e.logPath}`);
            }
            if (e instanceof errors_1.BaseCodestoreError) {
                this.log(e.message);
                return;
            }
            if ((e === null || e === void 0 ? void 0 : e.constructor.name) === 'RequiredArgsError') {
                this.log(e.message);
                return;
            }
            if (joi_1.default.isError(e)) {
                this.log(`${chalk_1.red('You have invalid')} ${chalk_1.yellow('codestore.yaml')} ${chalk_1.red('file:')}\n\t%s`, e.details.map((d) => d.context && d.message.replace(new RegExp(`^"${d.context.label}"`), chalk_1.bold(d.context.label))).join('\n\t'));
                return;
            }
            this.error(e.message || e.toString());
        }
        await this.verboseLogger.pushToFile();
    }
    async refreshToken() {
        let tokens;
        try {
            await this.setupApiClient(true); // it prevents sending auth header on refresh mutation
            const { refreshToken } = await this.homeFolderService.getTokens();
            tokens = await this.codestore.Auth.refreshToken(refreshToken, this.config.userAgent);
            await this.homeFolderService.saveTokens(tokens.accessToken, tokens.refreshToken);
        }
        catch (error) {
            await this.homeFolderService.removeTokens();
        }
        return tokens;
    }
    async setupApiClient(onLogin = false) {
        const commandName = this.id || 'undefined command';
        this.gqlClient = new apollo_boost_1.default({
            fetch: cross_fetch_1.default,
            // does not work when uri gets from config in terminal, should be rechecked
            uri: Command.getServiceUrl({ endpoint: 'federation-gateway-service' }),
            headers: {
                Authorization: onLogin ? '' : (this.authToken || (await this.homeFolderService.getTokens()).accessToken),
                'user-agent': this.config.userAgent,
                'x-cli-command': commandName,
            },
            onError: ({ graphQLErrors, operation, forward }) => {
                const isBadJWTError = graphQLErrors === null || graphQLErrors === void 0 ? void 0 : graphQLErrors.some(({ message }) => /Bad JWT/.test(message));
                if (!isBadJWTError)
                    return undefined;
                return utils_1.promiseToObservable(this.refreshToken()
                    .then((tokens) => {
                    operation.setContext(({ headers = {} }) => ({
                        headers: Object.assign(Object.assign({}, headers), { Authorization: tokens === null || tokens === void 0 ? void 0 : tokens.accessToken }),
                    }));
                    return forward(operation);
                }));
            },
        });
        this._codestore = new api_client_1.default(this.homeFolderService, this.gqlClient, this.verboseLogger.register(commandName));
    }
    // this method is required if we need do actions after login in one command, now it works to get token and receive user info
    async resetApiClient() {
        await this.setupApiClient();
    }
    renderTable(data, schema, options = { 'no-truncate': true }) {
        cli_ux_1.default.table(data, schema, options);
    }
    async findService(serviceName, inputServices) {
        const services = inputServices || await this.codestore.Service.list();
        if (!services.length) {
            this.log(`There are no services in the selected project. Try to add a new one using ${chalk_1.yellow('codestore project:service:add')} command.`);
            return undefined;
        }
        return services.find((service) => service.uniqueName === serviceName);
    }
    async chooseService(args, prefix = 'Choose a service', inputServices) {
        const services = inputServices || await this.codestore.Service.list();
        if (!services.length) {
            this.log(`There are no services in the selected project. Try to add a new one using ${chalk_1.yellow('codestore project:service:add')} command.`);
            return undefined;
        }
        const { serviceArg } = args;
        const foundService = await this.findService(serviceArg, services);
        if (foundService)
            return foundService;
        if (serviceArg) {
            this.log(`Service with id ${chalk_1.blue(serviceArg)} does not exist`);
        }
        const serviceMap = new Map(services.map((s) => [
            `${s.uniqueName}\t${s.problemSolving}`,
            s,
        ]));
        const { choosedService } = await inquirer_1.default.prompt({
            type: 'list',
            name: 'choosedService',
            message: 'Service:',
            prefix: utils_1.createPrefix(prefix),
            choices: Array.from(serviceMap.keys()),
        });
        this.log('');
        return serviceMap.get(choosedService);
    }
    async chooseVersion(service) {
        const versionsMap = new Map(service.versions
            .sort((a, b) => b.name.localeCompare(a.name))
            .map((v) => [`${v.name}: ${v.description}`, v]));
        const { version } = (await inquirer_1.default.prompt({
            type: 'list',
            name: 'version',
            message: 'Please select the version',
            choices: Array.from(versionsMap.keys()),
        }));
        this.log('');
        return versionsMap.get(version);
    }
    async chooseEnvironment(choices, message = 'Please select the environment') {
        const { env } = (await inquirer_1.default.prompt({
            type: 'list',
            name: 'env',
            message,
            choices: choices instanceof Map ? Array.from(choices.keys()) : choices,
        }));
        this.log('');
        return choices instanceof Map ? choices.get(env) : env;
    }
    async findProject(projectName, inputProjects) {
        const projects = inputProjects || await this.codestore.Project.list();
        if (!projects.length) {
            this.log(`There are no projects yet, try creating one using ${chalk_1.yellow('codestore project:create')} command.`);
            return undefined;
        }
        return projects.find((project) => project.uniqueName === projectName);
    }
    async chooseProject(args, prefix = 'Choose a project', withEmptySlot = false) {
        const projects = await this.codestore.Project.list();
        if (!projects.length) {
            this.log(`There are no projects yet, try creating one using ${chalk_1.yellow('codestore project:create')} command.`);
            return undefined;
        }
        const { projectArg } = args;
        const foundProject = await this.findProject(projectArg, projects);
        if (foundProject) {
            return foundProject;
        }
        if (projectArg) {
            this.log(`Project with id ${chalk_1.blue(projectArg)} does not exist`);
        }
        const mappedProjects = projects.map((s) => [
            `${s.uniqueName}\t${s.description}`,
            s,
        ]);
        if (withEmptySlot) {
            mappedProjects.push(['-', undefined]);
        }
        const projectMap = new Map(mappedProjects);
        const { choosedProject } = await inquirer_1.default.prompt({
            type: 'list',
            name: 'choosedProject',
            message: 'Project:',
            prefix: utils_1.createPrefix(prefix),
            choices: Array.from(projectMap.keys()),
        });
        this.log('');
        return projectMap.get(choosedProject);
    }
    static getServiceUrl({ endpoint }) {
        if (endpoint !== 'federation-gateway-service') {
            return `${this.apiPath}/${endpoint.split('-').join('')}/graphql`;
        }
        return `${this.apiPath}/${endpoint}/graphql`;
    }
    async validate(fieldName, fieldValue, flagsData) {
        return this.rules[fieldName] ? this.rules[fieldName](fieldValue, flagsData) : true;
    }
    async validateFlags(flagsOptions, flagsData) {
        const result = {};
        const flagKeys = Object.keys(flagsOptions);
        await Promise.all(flagKeys.map(async (flagName) => {
            const validationResult = await this.validate(flagName, flagsData[flagName], flagsData);
            if (validationResult !== true)
                this.error(validationResult);
            result[flagName] = flagsData[flagName];
        }));
        return result;
    }
    flagsLength(flagsData) {
        const flagNames = Object.keys(flagsData).filter((flagName) => flagName !== 'verbose');
        return !!flagNames.length;
    }
}
exports.default = Command;
Command.flags = {
    verbose: command_1.flags.boolean({
        description: 'Logs all the requests',
    }),
    verbosePath: command_1.flags.string({
        description: 'Path to store logs',
        dependsOn: ['verbose'],
    }),
};
Command.apiPath = process.env.CODESTORE_GATEWAY_HOST || 'https://api.code.store';
Command.appPath = process.env.CODESTORE_APP_HOST || 'https://app.code.store';
