"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = require("path");
const os_1 = require("os");
const codestore_utils_1 = require("codestore-utils");
const home_file_names_1 = tslib_1.__importDefault(require("../common/constants/home.file.names"));
const home_folder_names_1 = tslib_1.__importDefault(require("../common/constants/home.folder.names"));
const errors_1 = require("./errors");
const logFiles_1 = tslib_1.__importDefault(require("../common/constants/logFiles"));
class HomeFolderService {
    constructor() {
        this.homePath = path_1.join(os_1.homedir(), home_folder_names_1.default.CODESTORE);
        this.accessTokenPath = path_1.join(this.homePath, home_file_names_1.default.ACCESS_TOKEN);
        this.refreshTokenPath = path_1.join(this.homePath, home_file_names_1.default.REFRESH_TOKEN);
        this.homeLogsPath = path_1.join(this.homePath, logFiles_1.default.VERBOSE);
        this.fs = codestore_utils_1.PromisifiedFs;
    }
    async isHomeFolderExists() {
        try {
            const stats = await this.fs.stat(this.homePath);
            return stats.isDirectory();
        }
        catch (e) {
            return false;
        }
    }
    async createHomeFolder() {
        await this.fs.mkdir(this.homePath);
        await this.writeFileInHomeFolder('credentials', '');
    }
    async writeFileInHomeFolder(filename, data) {
        const path = path_1.join(this.homePath, filename);
        return this.fs.writeFile(path, data);
    }
    async saveTokens(accessToken, refreshToken) {
        if (!await this.isHomeFolderExists()) {
            await this.createHomeFolder();
        }
        await this.writeFileInHomeFolder(home_file_names_1.default.ACCESS_TOKEN, accessToken);
        return this.writeFileInHomeFolder(home_file_names_1.default.REFRESH_TOKEN, refreshToken);
    }
    async removeTokens() {
        if (!await this.isHomeFolderExists()) {
            await this.createHomeFolder();
        }
        await this.writeFileInHomeFolder(home_file_names_1.default.ACCESS_TOKEN, '');
        return this.writeFileInHomeFolder(home_file_names_1.default.REFRESH_TOKEN, '');
    }
    async getTokens() {
        if (!await this.isHomeFolderExists()
            || await this.isTokenFileEmpty(this.accessTokenPath)
            || await this.isTokenFileEmpty(this.refreshTokenPath)) {
            throw new errors_1.NotAuthorizedError();
        }
        const accessToken = await this.getToken(this.accessTokenPath);
        const refreshToken = await this.getToken(this.refreshTokenPath);
        return { accessToken, refreshToken };
    }
    async getToken(path) {
        var _a, _b;
        return (_b = (_a = (await this.fs.readFile(path))) === null || _a === void 0 ? void 0 : _a.toString()) === null || _b === void 0 ? void 0 : _b.replace(/\n$/, '');
    }
    async isTokenFileEmpty(tokenPath) {
        try {
            const stats = await this.fs.stat(tokenPath);
            if (!stats.isFile())
                return true;
            const f = await this.fs.readFile(tokenPath);
            return f.toString() === '';
        }
        catch (e) {
            return false;
        }
    }
    get logsPath() {
        return this.homeLogsPath;
    }
}
exports.default = HomeFolderService;
