"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const yaml_1 = require("yaml");
const chalk_1 = require("chalk");
const codestore_utils_1 = require("codestore-utils");
const codestore_config_validate_1 = require("./codestore-config-validate");
const errors_1 = require("./errors");
class ServiceWorker {
    constructor() {
        this.configFiles = {
            codestore: {
                path: codestore_utils_1.PathsResolverTool.YAML,
                error: new errors_1.WrongFolderError(`You must be in code.store service root folder to invoke this command.
Check if codestore.yaml and schema.graphql exist.`),
            },
            schema: {
                path: codestore_utils_1.PathsResolverTool.SCHEMA,
                error: new errors_1.WrongFolderError(`Cannot find schema.graphql, make sure that you invoke this command from the service root folder or use ${chalk_1.yellow('cs service:pull')} to restore your graphql.schema.`),
            },
        };
    }
    async load(configName) {
        const { path, error } = this.configFiles[configName];
        try {
            await codestore_utils_1.PromisifiedFs.access(path);
            return path;
        }
        catch (e) {
            throw error;
        }
    }
    async loadValuesFromYaml() {
        const configPath = await this.load('codestore');
        const file = await codestore_utils_1.PromisifiedFs.readFile(configPath);
        return codestore_config_validate_1.validateCodestoreConfig(yaml_1.parse(file.toString()));
    }
    static logPath(fileName) {
        return path_1.join(codestore_utils_1.PathsResolverTool.ROOT, 'logs', fileName || '');
    }
    static async writeLog(fileName, log) {
        await codestore_utils_1.PromisifiedFs.createFolderIfNotExist(this.logPath());
        const file = this.logPath(fileName);
        await codestore_utils_1.PromisifiedFs.writeFile(file, log);
        return file;
    }
    overwriteYaml(data) {
        return codestore_utils_1.PromisifiedFs.writeFile(codestore_utils_1.PathsResolverTool.YAML, data);
    }
    async applyMiddleware(endpoint, position, name) {
        var _a;
        const data = await this.loadValuesFromYaml();
        const { rest } = data;
        if (!rest || !((_a = rest.routes) === null || _a === void 0 ? void 0 : _a.find((route) => route.path === endpoint))) {
            throw new Error(`Route ${endpoint} not found, please check codestore.yaml`);
        }
        data.rest.routes = rest.routes.map((route) => {
            const middlewareKey = `${position}Middlewares`;
            if (route.path === endpoint) {
                return Object.assign(Object.assign({}, route), { [middlewareKey]: [
                        ...route[middlewareKey] || [],
                        name,
                    ] });
            }
            return route;
        });
        await this.overwriteYaml(yaml_1.stringify(data));
    }
}
exports.default = ServiceWorker;
