"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const is_installed_globally_1 = tslib_1.__importDefault(require("is-installed-globally"));
const has_yarn_1 = tslib_1.__importDefault(require("has-yarn"));
const is_yarn_global_1 = tslib_1.__importDefault(require("is-yarn-global"));
const boxen_1 = tslib_1.__importDefault(require("boxen"));
const is_ci_1 = tslib_1.__importDefault(require("is-ci"));
const codestore_utils_1 = require("codestore-utils");
const semver_diff_1 = tslib_1.__importDefault(require("semver-diff"));
const child_process_1 = require("child_process");
const path_1 = tslib_1.__importDefault(require("path"));
const update_checker_1 = tslib_1.__importDefault(require("../../common/constants/update-checker"));
const debug = require('debug')('update-checker:class');
class UpdateChecker {
    constructor(versionFilePath, currentVersion) {
        this.versionFilePath = versionFilePath;
        this.currentVersion = currentVersion;
        this.latestVersion = currentVersion;
    }
    async check() {
        try {
            debug('checking the updates');
            if (process.argv[2] === 'update' || process.env.NODE_ENV === 'test' || is_ci_1.default || !process.stdout.isTTY)
                return;
            debug(`reading local version file ${this.versionFilePath}`);
            const localDistTags = JSON.parse(await codestore_utils_1.PromisifiedFs.readFile(this.versionFilePath, 'utf8'));
            this.latestVersion = localDistTags.latest;
            const versionDiff = semver_diff_1.default(this.currentVersion, this.latestVersion);
            if (versionDiff && versionDiff !== 'latest') {
                this.notify();
            }
        }
        catch (error) {
            if (error.code !== 'ENOENT')
                throw error;
        }
        if (await this.refreshNeeded())
            await this.spawnRefresh();
    }
    async refreshNeeded() {
        try {
            const { mtime } = await codestore_utils_1.PromisifiedFs.stat(this.versionFilePath);
            const staleAt = new Date(mtime.valueOf() + (1000 * 60 * 60 * update_checker_1.default.TIMEOUT_IN_HOURS));
            return staleAt < new Date();
        }
        catch (error) {
            debug(error);
            return true;
        }
    }
    async spawnRefresh() {
        debug('spawning a detached process, passing options as env arguments');
        debug(`spawning at: ${path_1.default.join(__dirname, '../../lib/update-checker/get-version')}`);
        child_process_1.spawn(process.execPath, [path_1.default.join(__dirname, '../../lib/update-checker/get-version'), this.versionFilePath, this.currentVersion], {
            detached: true,
            windowsHide: true,
            stdio: 'ignore',
        }).unref();
    }
    notify() {
        let installMsg;
        // TODO: check for standalone & brew installation methods
        if (is_yarn_global_1.default()) {
            installMsg = `Run ${chalk_1.default.cyan('yarn global add codestore')}`;
        }
        else if (is_installed_globally_1.default) {
            installMsg = `Run ${chalk_1.default.cyan('npm i -g codestore')}`;
        }
        else if (has_yarn_1.default()) {
            installMsg = `Run ${chalk_1.default.cyan('yarn add codestore')}`;
        }
        else {
            installMsg = `Run ${chalk_1.default.cyan('npm i -g codestore')}
      or ${chalk_1.default.cyan('brew upgrade codestore')}
      depending on your installation method`;
        }
        const template = `Update available ${chalk_1.default.dim(this.currentVersion)} ${chalk_1.default.reset('→')} ${chalk_1.default.green(this.latestVersion)}
    ${installMsg}`;
        // TODO: enable borderStyle: 'round'
        const message = boxen_1.default(template, {
            padding: 1,
            margin: 1,
            align: 'center',
            borderColor: 'yellow',
        });
        process.stderr.write(message);
        process.stderr.write('\n');
    }
}
exports.default = UpdateChecker;
